import * as _ from 'lodash';

import * as Promise from 'bluebird';
import * as isAfter from 'date-fns/is_after';
import * as addMilliseconds from 'date-fns/add_milliseconds';

import {TokenRequestData} from './utils/TokenExchange';

import {Factory as ClientConfigFactory} from './utils/ClientConfigFactory';
import {RequestManager} from './RequestManager';
import {ClientError} from './utils/ClientError';

import {
  AccountPlanResource,
  AccountSubscriptionResource,
  AddressResource,
  BannerResource,
  CartResource,
  CartItemResource,
  ClinicResource,
  EventResource,
  LicenseResource,
  NetworkFileResource,
  OrderResource,
  OrderAddressResource,
  OrderItemResource,
  PatientResource,
  PatientAddressResource,
  PatientAllergyResource,
  PlanResource,
  PrescriptionResource,
  PrescriptionCartResource,
  PrescriptionCartItemResource,
  PricingRuleResource,
  ProductResource,
  ProductAttributeResource,
  ProductBrandResource,
  ProductBrandMediaResource,
  ProductBundleResource,
  ProductBundleItemResource,
  ProductCategoryResource,
  ProductClickResource,
  ProductMediaResource,
  ProductShippingSettingResource,
  ProductSubcategoryResource,
  ProfileResource,
  ProfileEmailResource,
  ProfileLicenseResource,
  ProfileVendorContactResource,
  PromoCodeResource,
  ResaleCertificateResource,
  ReviewResource,
  RoleResource,
  SalesAccountResource,
  SalesAccountAddressResource,
  SalesAccountContactResource,
  ShippingResource,
  SkuResource,
  SkuProductAttributeDescriptionResource,
  StateTaxableProductCategoryExemptionRuleResource,
  SubscriptionResource,
  TaxableProductCategoryResource,
  TaxExemptionRuleResource,
  TrainingResource,
  TrainingCategoryResource,
  TrainingCompletionResource,
  TrainingLocationResource,
  TrainingLectureResource,
  TrainingMaterialResource,
  TrainingModuleResource,
  TrainingQuizResource,
  TrainingQuestionResource,
  TrainingAnswerResource,
  VendorResource,
  VendorAddressResource,
  VendorContactResource,
  VendorShippingSettingResource,
  VendorOrderRuleResource,
  TrainingRatingsResource,
  TrainingDiscountsResource,
  TrainingStatisticsResource,
  EnvironmentSettingsResource,
  ProductDiscountResource,
  ProductDiscountThresholdResource,
  CombinedDiscountResource,
  CombinedDiscountItemResource
} from './resources';

export {
    AccountPlan,
    AccountSubscription,
    Address,
    Banner,
    Cart,
    CartItem,
    Clinic,
    Event,
    License,
    NetworkFile,
    Order,
    OrderAddress,
    OrderItem,
    Patient,
    PatientAddress,
    PatientAllergy,
    Plan,
    Prescription,
    PrescriptionCart,
    PrescriptionCartItem,
    PricingRule,
    Product,
    ProductAttribute,
    ProductBrand,
    ProductBrandMedia,
    ProductBundle,
    ProductBundleItem,
    ProductCategory,
    ProductClick,
    ProductMedia,
    ProductShippingSetting,
    ProductSubcategory,
    Profile,
    ProfileEmail,
    ProfileLicense,
    ProfileVendorContact,
    PromoCode,
    ResaleCertificate,
    Review,
    Role,
    SalesAccount,
    SalesAccountAddress,
    SalesAccountContact,
    Sku,
    SkuProductAttributeDescription,
    StateTaxableProductCategoryExemptionRule,
    Subscription,
    TaxableProductCategory,
    TaxExemptionRule,
    Training,
    TrainingCategory,
    TrainingCompletion,
    TrainingLocation,
    TrainingLecture,
    TrainingMaterial,
    TrainingQuestion,
    TrainingAnswer,
    Vendor,
    VendorAddress,
    VendorContact,
    VendorOrderRule,
    VendorShippingSetting,
    RateRecord,
    EnvironmentSettings,
    ProductDiscount,
    ProductDiscountThreshold,
    CombinedDiscount,
    CombinedDiscountItem
} from './resources';


export interface ClientConfiguration {
    clientId: string;
    clientSecret?: string;
    redirectUri?: string;
    baseApiUrl?: string;
    baseAuthUrl?: string;
    request?: object;
}

export class Client {
    public baseApiUrl: string;
    public accessToken?: string;
    public refreshToken?: string;
    public tokenExpiresAt?: Date;

    public accountPlan: AccountPlanResource;
    public accountSubscription: AccountSubscriptionResource;
    public address: AddressResource;
    public banner: BannerResource;
    public cart: CartResource;
    public cartItem: CartItemResource;
    public clinic: ClinicResource;
    public event: EventResource;
    public license: LicenseResource;
    public networkFile: NetworkFileResource;
    public order: OrderResource;
    public orderAddress: OrderAddressResource;
    public orderItem: OrderItemResource;
    public patient: PatientResource;
    public patientAddress: PatientAddressResource;
    public patientAllergy: PatientAllergyResource;
    public plan: PlanResource;
    public prescription: PrescriptionResource;
    public prescriptionCart: PrescriptionCartResource;
    public prescriptionCartItem: PrescriptionCartItemResource;
    public pricingRule: PricingRuleResource;
    public product: ProductResource;
    public productAttribute: ProductAttributeResource;
    public productBrand: ProductBrandResource;
    public productBrandMedia: ProductBrandMediaResource;
    public productBundle: ProductBundleResource;
    public productBundleItem: ProductBundleItemResource;
    public productCategory: ProductCategoryResource;
    public productClick: ProductClickResource;
    public productMedia: ProductMediaResource;
    public productShippingSetting: ProductShippingSettingResource;
    public productSubcategory: ProductSubcategoryResource;
    public profile: ProfileResource;
    public profileEmail: ProfileEmailResource;
    public profileLicense: ProfileLicenseResource;
    public profileVendorContact: ProfileVendorContactResource;
    public promoCode: PromoCodeResource;
    public resaleCertificate: ResaleCertificateResource;
    public review: ReviewResource;
    public role: RoleResource;
    public salesAccount: SalesAccountResource;
    public salesAccountAddress: SalesAccountAddressResource;
    public salesAccountContact: SalesAccountContactResource;
    public shipping: ShippingResource;
    public sku: SkuResource;
    public skuProductAttributeDescription: SkuProductAttributeDescriptionResource;
    public stateTaxableProductCategoryExemptionRule: StateTaxableProductCategoryExemptionRuleResource;
    public subscription: SubscriptionResource;
    public taxableProductCategory: TaxableProductCategoryResource;
    public taxExemptionRule: TaxExemptionRuleResource;
    public training: TrainingResource;
    public trainingCategory: TrainingCategoryResource;
    public trainingCompletion: TrainingCompletionResource;
    public trainingLocation: TrainingLocationResource;
    public trainingLecture: TrainingLectureResource;
    public trainingMaterial: TrainingMaterialResource;
    public trainingModule: TrainingModuleResource;
    public trainingQuiz: TrainingQuizResource;
    public trainingQuestion: TrainingQuestionResource;
    public trainingAnswer: TrainingAnswerResource;
    public trainingStatistics: TrainingStatisticsResource;
    public vendor: VendorResource;
    public vendorAddress: VendorAddressResource;
    public vendorContact: VendorContactResource;
    public vendorOrderRule: VendorOrderRuleResource;
    public vendorShippingSetting: VendorShippingSettingResource;
    public trainingRatings: TrainingRatingsResource;
    public trainingDiscounts: TrainingDiscountsResource;
    public environmentSettings: EnvironmentSettingsResource;
    public productDiscount: ProductDiscountResource;
    public productDiscountThreshold: ProductDiscountThresholdResource;
    public combinedDiscount: CombinedDiscountResource;
    public combinedDiscountItem: CombinedDiscountItemResource;

    private authorizationCode?: string;
    private baseAuthUrl: string;
    private clientId: string;
    private clientSecret?: string;
    private redirectUri?: string;

    private requestManager: RequestManager;

    public static create(clientConfig: ClientConfiguration) {
        const config = ClientConfigFactory(clientConfig);
        return new Client(config);
    }

    constructor(config: ClientConfiguration) {
        this.baseApiUrl = config.baseApiUrl || 'https://api.ylift.com/v1';
        this.baseAuthUrl = config.baseAuthUrl || 'https://accounts.ylift.com';
        if (config.redirectUri) this.redirectUri = config.redirectUri;

        this.clientId = config.clientId;
        if (config.clientSecret) this.clientSecret = config.clientSecret;

        this.requestManager = new RequestManager(this, config);

        this.accountPlan = new AccountPlanResource(this.requestManager);
        this.accountSubscription = new AccountSubscriptionResource(this.requestManager);
        this.address = new AddressResource(this.requestManager);
        this.banner = new BannerResource(this.requestManager);
        this.cart = new CartResource(this.requestManager);
        this.cartItem = new CartItemResource(this.requestManager);
        this.clinic = new ClinicResource(this.requestManager);
        this.event = new EventResource(this.requestManager);
        this.license = new LicenseResource(this.requestManager);
        this.networkFile = new NetworkFileResource(this.requestManager);
        this.order = new OrderResource(this.requestManager);
        this.orderAddress = new OrderAddressResource(this.requestManager);
        this.orderItem = new OrderItemResource(this.requestManager);
        this.patient = new PatientResource(this.requestManager);
        this.patientAddress = new PatientAddressResource(this.requestManager);
        this.patientAllergy = new PatientAllergyResource(this.requestManager);
        this.plan = new PlanResource(this.requestManager);
        this.prescription = new PrescriptionResource(this.requestManager);
        this.prescriptionCart = new PrescriptionCartResource(this.requestManager);
        this.prescriptionCartItem = new PrescriptionCartItemResource(this.requestManager);
        this.pricingRule = new PricingRuleResource(this.requestManager);
        this.product = new ProductResource(this.requestManager);
        this.productAttribute = new ProductAttributeResource(this.requestManager);
        this.productBrand = new ProductBrandResource(this.requestManager);
        this.productBrandMedia = new ProductBrandMediaResource(this.requestManager);
        this.productBundle = new ProductBundleResource(this.requestManager);
        this.productBundleItem = new ProductBundleItemResource(this.requestManager);
        this.productCategory = new ProductCategoryResource(this.requestManager);
        this.productClick = new ProductClickResource(this.requestManager);
        this.productMedia = new ProductMediaResource(this.requestManager);
        this.productShippingSetting = new ProductShippingSettingResource(this.requestManager);
        this.productSubcategory = new ProductSubcategoryResource(this.requestManager);
        this.profile = new ProfileResource(this.requestManager);
        this.profileEmail = new ProfileEmailResource(this.requestManager);
        this.profileLicense = new ProfileLicenseResource(this.requestManager);
        this.profileVendorContact = new ProfileVendorContactResource(this.requestManager);
        this.promoCode = new PromoCodeResource(this.requestManager);
        this.resaleCertificate = new ResaleCertificateResource(this.requestManager);
        this.review = new ReviewResource(this.requestManager);
        this.role = new RoleResource(this.requestManager);
        this.salesAccount = new SalesAccountResource(this.requestManager);
        this.salesAccountAddress = new SalesAccountAddressResource(this.requestManager);
        this.salesAccountContact = new SalesAccountContactResource(this.requestManager);
        this.shipping = new ShippingResource(this.requestManager);
        this.sku = new SkuResource(this.requestManager);
        this.skuProductAttributeDescription = new SkuProductAttributeDescriptionResource(this.requestManager);
        this.stateTaxableProductCategoryExemptionRule = new StateTaxableProductCategoryExemptionRuleResource(this.requestManager);
        this.subscription = new SubscriptionResource(this.requestManager);
        this.taxableProductCategory = new TaxableProductCategoryResource(this.requestManager);
        this.taxExemptionRule = new TaxExemptionRuleResource(this.requestManager);
        this.training = new TrainingResource(this.requestManager);
        this.trainingCategory = new TrainingCategoryResource(this.requestManager);
        this.trainingCompletion = new TrainingCompletionResource(this.requestManager);
        this.trainingLocation = new TrainingLocationResource(this.requestManager);
        this.trainingLecture = new TrainingLectureResource(this.requestManager);
        this.trainingStatistics = new TrainingStatisticsResource(this.requestManager);
        this.trainingMaterial = new TrainingMaterialResource(this.requestManager);
        this.trainingModule = new TrainingModuleResource(this.requestManager);
        this.trainingQuestion = new TrainingQuestionResource(this.requestManager);
        this.trainingAnswer = new TrainingAnswerResource(this.requestManager);
        this.trainingQuiz = new TrainingQuizResource(this.requestManager);
        this.vendor = new VendorResource(this.requestManager);
        this.vendorAddress = new VendorAddressResource(this.requestManager);
        this.vendorContact = new VendorContactResource(this.requestManager);
        this.vendorOrderRule = new VendorOrderRuleResource(this.requestManager);
        this.vendorShippingSetting = new VendorShippingSettingResource(this.requestManager);
        this.trainingRatings = new TrainingRatingsResource(this.requestManager);
        this.trainingDiscounts = new TrainingDiscountsResource(this.requestManager);
        this.environmentSettings = new EnvironmentSettingsResource(this.requestManager);
        this.productDiscount = new ProductDiscountResource(this.requestManager);
        this.productDiscountThreshold = new ProductDiscountThresholdResource(this.requestManager);
        this.combinedDiscount = new CombinedDiscountResource(this.requestManager);
        this.combinedDiscountItem = new CombinedDiscountItemResource(this.requestManager);
    }

    // Defaults to the more secure 'code' option
    public getAuthorizeUri(responseType: string = 'code'): string {
        if (!this.redirectUri) throw new ClientError('cannot generate authorization URL without redirectUri');

        const params = _.map({
            response_type: responseType,
            client_id: this.clientId,
            redirect_uri: this.redirectUri,
        }, (value, key) => `${key}=${encodeURIComponent(value)}`).join('&');

        return `${this.baseAuthUrl}/oauth2/authorize?${params}`
    }

    public getLogoutUri(): string {
        if (!this.redirectUri) throw new ClientError('cannot generate logout URL without redirectUri');

        return `${this.baseAuthUrl}/logout/?redirectTo=${this.redirectUri}`
    }

    public refreshTokens(): Promise<void> {
        if (this.tokenExpiresAt && isAfter(this.tokenExpiresAt, new Date()))
            return Promise.resolve();

        if (this.authorizationCode) return this.exchangeAuthCode();

        if (this.refreshToken) return this.handleTokenRefresh();

        if (this.clientSecret) return this.exchangeClientCredentials();

        return Promise.resolve();
    }

    public setAccessToken(accessToken: string) {
        this.accessToken = accessToken;
    }

    public setAuthorizationCode(authorizationCode: string) {
        this.authorizationCode = authorizationCode;
    }

    // Private

    private exchangeAuthCode(): Promise<void> {
        return this.getTokens({
            grant_type: 'authorization_code',
            code: this.authorizationCode
        })
            .then(() => {
                delete this.authorizationCode;
            })
    }

    private exchangeClientCredentials(): Promise<void> {
        return this.getTokens({
            grant_type: 'client_credentials'
        })
    }

    private getTokens(data: TokenRequestData): Promise<void> {
        const payload = _.merge({
            client_id: this.clientId,
            client_secret: this.clientSecret,
        }, data);

        return this.requestManager.makeRequest({
            method: 'POST',
            data: payload,
            url: `${this.baseAuthUrl}/oauth2/token`
        }, {refreshTokens: false})
            .then((response) => {
                this.accessToken = response.access_token;
                if (response.refresh_token) this.refreshToken = response.refresh_token;
                this.tokenExpiresAt = addMilliseconds(new Date(), response.expires_in);
            });
    }

    private handleTokenRefresh(): Promise<void> {
        return this.getTokens({
            grant_type: 'refresh_grant',
            refresh_token: this.refreshToken
        })
    }
}
