import * as Promise from 'bluebird';
import * as _ from 'lodash';

import {
    YLIFTCreateAttributes,
    YLIFTListAttributes,
    YLIFTUpdateAttributes,
    Resource
} from '../utils/Resource';
import {RequestManager} from '../RequestManager';
import {CartItem} from "./CartItem";

export interface Cart {
    id: number;
    createdAt: Date;
    updatedAt: Date;
    profileId: number;
    cartItems?: CartItem[];
}

export interface CartOrderDataAttributes {
    id: number;
    zip?: string;
}

export interface CartGetProfileAttributes {
    profileId?: number;
}

export interface CartItemCreateAttributes extends YLIFTUpdateAttributes {
    quantity: number;
    skuId: number;
}

export interface CartItemUpdateAttributes extends YLIFTUpdateAttributes {
    id: number;
    quantity: number;
}

export class CartResource extends Resource<Cart, YLIFTListAttributes, YLIFTCreateAttributes, YLIFTUpdateAttributes> {
    constructor(requestManager: RequestManager) {
        super('/carts', requestManager);
    }

    public get(id: number): Promise<Cart> {
        return this._get(id);
    }

    public getByProfileId(params: CartGetProfileAttributes): Promise<Cart> {
        return this.makeRequest({
            method: 'GET',
            params: params,
            url: `${this.basePath}/profile`
        })
    }

    public getShippingSettings() {
        return this.makeRequest({
            method: 'GET',
            url: `${this.basePath}/shipping-settings`
        })
    }

    public getOrderData(params: CartOrderDataAttributes): Promise<Cart> {
        return this.makeRequest({
            method: 'GET',
            params: params,
            url: `${this.basePath}/order-info`
        })
    }

    public clear() {
        return this.makeRequest({
            method: 'POST',
            url: `${this.basePath}/clear`,
        })
    }

    // Cart Item Routes

    public createCartItem(createAttributes: CartItemCreateAttributes): Promise<Cart> {
        return this.makeRequest({
            method: 'POST',
            url: `${this.basePath}/items`,
            data: createAttributes
        })
    }

    public deleteCartItem(id: number): Promise<Cart> {
        return this.makeRequest({
            method: 'DELETE',
            params: {id},
            url: `${this.basePath}/items/{id}`
        })
    }

    public updateCartItem(updateAttributes: CartItemUpdateAttributes): Promise<Cart> {
        return this.makeRequest({
            method: 'PUT',
            params: _.pick(updateAttributes, ['id']),
            url: `${this.basePath}/items/{id}`,
            data: _.pick(updateAttributes, ['quantity'])
        })
    }

}
