import * as Promise from 'bluebird';

import {
    YLIFTCreateAttributes,
    YLIFTListAttributes,
    YLIFTUpdateAttributes,
    Resource
} from '../utils/Resource';
import { RequestManager } from '../RequestManager';

export interface NetworkFile {
    id: number;
    createdAt: Date;
    updatedAt: Date;
    title: string;
    caption: string|null;
    description: string|null;
    url: string;
    mimeType: string|null;
    views: number;
    size: number;
    isDownloadable: boolean|null;
    thumbnailUrl: string|null;
}

export interface NetworkFileTag {
    networkFileId: number;
    tag: string;
    createdAt: Date;
    updatedAt: Date;
}

export interface NetworkFileListAttributes extends YLIFTListAttributes {
    productBrandId?: number;
    tags?: string[];
    query?: string;
}

export interface NetworkFileCreateAttributes extends YLIFTCreateAttributes {
    title: string;
    caption?: string;
    description?: string;
    url: string;
    mimeType?: string;
    views: number;
    size: number;
    isDownloadable?: boolean;
    thumbnailUrl?: string;
}

export interface NetworkFileUpdateAttributes extends YLIFTUpdateAttributes {
    title?: string;
    caption?: string;
    description?: string;
    url?: string;
    mimeType?: string;
    views?: number;
    size?: number;
    isDownloadable?: boolean;
    thumbnailUrl?: string;
}

export interface NetworkFileUpdateBrandsAttributes {
    productBrandIds: number[];
}

export interface NetworkFileUpdateTrainingsAttributes {
    trainingIds: string[];
}

export interface NetworkFileUpdateProductsAttributes {
    productIds: number[];
}

export interface NetworkFileUpdateRolesAttributes {
    roles: string[];
}

export interface NetworkFileUpdateTagsAttributes {
    tags: string[];
}

export class NetworkFileResource extends Resource<NetworkFile, NetworkFileListAttributes, NetworkFileCreateAttributes, NetworkFileUpdateAttributes> {
    constructor(requestManager: RequestManager) {
        super('/network-files', requestManager);
    }

    public list(listArgs: NetworkFileListAttributes): Promise<{ count: number, rows: NetworkFile[] }> {
        return this._list(listArgs);
    }

    public listTags(): Promise<{ count: number, rows: NetworkFileTag[] }> {
        return this.makeRequest({
            method: 'GET',
            url: `${this.basePath}/tags`
        })
    }

    public create(createArgs: NetworkFileCreateAttributes): Promise<NetworkFile> {
        return this._create(createArgs);
    }

    public get(id: string | number): Promise<NetworkFile> {
        return this._get(id);
    }

    public update(id: string | number, updateArgs: NetworkFileUpdateAttributes): Promise<NetworkFile> {
        return this._update(id, updateArgs);
    }

    public updateBrands(id: string | number, updateBrandsArgs: NetworkFileUpdateBrandsAttributes): Promise<NetworkFile> {
        return this.makeRequest({
            method: 'PUT',
            data: updateBrandsArgs,
            url: `${this.basePath}/${id}/brands`
        })
    }

  public updateTrainings(id: string | number, updateTrainingArgs: NetworkFileUpdateTrainingsAttributes): Promise<NetworkFile> {
    return this.makeRequest({
      method: 'PUT',
      data: updateTrainingArgs,
      url: `${this.basePath}/${id}/trainings`
    })
  }

    public updateProducts(id: string | number, updateProductsArgs: NetworkFileUpdateProductsAttributes): Promise<NetworkFile> {
        return this.makeRequest({
            method: 'PUT',
            data: updateProductsArgs,
            url: `${this.basePath}/${id}/products`
        })
    }

    public updateRoles(id: string | number, updateRolesArgs: NetworkFileUpdateRolesAttributes): Promise<NetworkFile> {
        return this.makeRequest({
            method: 'PUT',
            data: updateRolesArgs,
            url: `${this.basePath}/${id}/roles`
        })
    }

    public updateTags(id: string | number, updateTagsArgs: NetworkFileUpdateTagsAttributes): Promise<NetworkFile> {
        return this.makeRequest({
            method: 'PUT',
            data: updateTagsArgs,
            url: `${this.basePath}/${id}/tags`
        })
    }

    public del(id: string | number): Promise<void> {
        return this.destroy(id);
    }

    public destroy(id: string | number): Promise<void> {
        return this._destroy(id);
    }
}
