import * as Promise from 'bluebird';
import {head} from 'lodash';

import {
    YLIFTCreateAttributes,
    YLIFTListAttributes,
    YLIFTUpdateAttributes,
    Resource
} from '../utils/Resource';
import { RequestManager } from '../RequestManager';
import {OrderItem} from "./OrderItem";
import {OrderAddress} from './OrderAddress';

export interface Order {
    id: number;
    createdAt: Date;
    updatedAt: Date;
    status: 'PENDING'|'COMPLETED'|'AWAITING_APPROVAL'|'CANCELED';
    orderItems?: OrderItem[];
    orderAddress?: OrderAddress;
    amount: number;
    canceledAt?: Date;
    approvedAt?: Date;
    completedAt?: Date;
    cardId: string;
    customerId: string;
    orderAddressId: number;
    label: string;
    eventId?: string;
    subscriptionId?: number;
    prescriptionId?: number;
}

export interface OrderGetAttributes {
    id: number;
    profileId: number;
}

export interface OrderListAttributes extends YLIFTListAttributes {
    status?: 'PENDING'|'COMPLETED'|'AWAITING_APPROVAL'|'CANCELED';
    cardId?: string;
    customerId?: string;
    orderAddressId?: number;
    label?: string;
    subscriptionId?: number;
    prescriptionId?: number;
    profileId?: number;
    productId?: number;
    createdBefore?: Date;
    createdAfter?: Date;
    id?: number;
}

export interface OrderSearchAttributes extends YLIFTListAttributes {
    id?: number;
    profileId?: number;
    sort?: string;
    price?: string;
}

export interface OrderCreateAttributes extends YLIFTCreateAttributes {
    cardId: string;
    addressId: string;
}

export interface OrderUpdateAttributes extends YLIFTUpdateAttributes {
    status: 'PENDING'|'COMPLETED'|'AWAITING_APPROVAL'|'CANCELED';
}

export interface LeaseCreateAttributes {
    companyInfo: {
        name: string;
        yearsInBusiness: number;
        businessType: 'Proprietorship' | 'Partnership' | 'Corporation' | 'Other';
        address: string;
        line2?: string;
        city: string;
        state: string;
        zip: string;
        phone: string;
        fax?: string;
        email: string;
        taxId: string;
        grossRevenue: number;
        personalIncome: number;
    }
    leaseInfo: {
        termLength: '36 Months' | '48 Months' | '60 Months';
    }
    personalInfo: {
        firstName: string;
        lastName: string;
        phone: string;
        licenseNumber: string;
        line1: string;
        line2?: string;
        city: string;
        state: string;
        zip: string;
        ssn: string;
        dob: Date;
    }
    productInfo: {
        skuId: number;
        quantity: number;
    }
}

export class OrderResource extends Resource<Order, OrderListAttributes, OrderCreateAttributes, OrderUpdateAttributes> {
    constructor(requestManager: RequestManager) {
        super('/orders', requestManager);
    }

    public search(searchArgs: OrderSearchAttributes): Promise<{ count: number, rows: Order[] }> {
        return this.makeRequest({
            method: 'GET',
            params: searchArgs,
            url: `${this.basePath}/search`
        })
    }

    public list(listArgs: OrderListAttributes): Promise<{ count: number, rows: Order[] }> {
        return this._list(listArgs);
    }

    public create(createArgs: OrderCreateAttributes): Promise<Order> {
        return this._create(createArgs);
    }

    public applyForLease(createArgs: LeaseCreateAttributes): Promise<Order> {
        return this.makeRequest({
            method: 'POST',
            data: createArgs,
            url: `${this.basePath}/lease`
        })
    }

    public get(getArgs: OrderGetAttributes): Promise<Order> {
        return this._list(getArgs)
            .then(({ rows: orders, count }) => {
                return head(orders);
            })
    }

    public update(id: string | number, updateArgs: OrderUpdateAttributes): Promise<Order> {
        return this._update(id, updateArgs);
    }

    public del(id: string | number): Promise<void> {
        return this.destroy(id);
    }

    public destroy(id: string | number): Promise<void> {
        return this._destroy(id);
    }
}
