import * as Promise from 'bluebird';
import * as _ from 'lodash';

import {
    YLIFTCreateAttributes,
    YLIFTListAttributes,
    YLIFTUpdateAttributes,
    Resource
} from '../utils/Resource';
import {RequestManager} from '../RequestManager';
import {PrescriptionCartItem} from "./PrescriptionCartItem";
import {Cart, CartGetProfileAttributes} from "./Cart";

export interface PrescriptionCart {
    id: number;
    createdAt: Date;
    updatedAt: Date;
    profileId: number;
    prescriptionCartItems?: PrescriptionCartItem[];
}

export interface PrescriptionCartOrderDataAttributes {
    id: number;
    zip?: string;
}

export interface PrescriptionCartItemCreateAttributes extends YLIFTUpdateAttributes {
    quantity: number;
    skuId: number;
    directions?: string;
}

export interface PrescriptionCartItemUpdateAttributes extends YLIFTUpdateAttributes {
    id: number;
    quantity?: number;
    directions?: string;
}

export interface PrescriptionCartUpdateAttributes extends YLIFTUpdateAttributes {
    clinicId?: number;
    patientId?: number;
    shipTo?: 'clinic' | 'patient';
    shippingMethod?: 'REGULAR' | 'EXPRESS' | 'OVERNIGHT';
}

export class PrescriptionCartResource extends Resource<PrescriptionCart, YLIFTListAttributes, YLIFTCreateAttributes, YLIFTUpdateAttributes> {
    constructor(requestManager: RequestManager) {
        super('/prescription-carts', requestManager);
    }

    public get(id: number): Promise<PrescriptionCart> {
        return this._get(id);
    }

    public getByProfileId(params: CartGetProfileAttributes): Promise<Cart> {
        return this.makeRequest({
            method: 'GET',
            params: params,
            url: `${this.basePath}/profile`
        })
    }

    public getOrderData(params: PrescriptionCartOrderDataAttributes): Promise<PrescriptionCart> {
        return this.makeRequest({
            method: 'GET',
            params: params,
            url: `${this.basePath}/{id}/order-info`
        })
    }

    public getShippingSettings() {
        return this.makeRequest({
            method: 'GET',
            url: `${this.basePath}/shipping-settings`
        })
    }

    public update(updateArgs: PrescriptionCartUpdateAttributes): Promise<PrescriptionCart> {
        return this.makeRequest({
            method: 'PUT',
            url: `${this.basePath}`,
            data: updateArgs
        })
    }

    // Prescription Cart Item Routes

    public createCartItem(createAttributes: PrescriptionCartItemCreateAttributes): Promise<PrescriptionCart> {
        return this.makeRequest({
            method: 'POST',
            url: `${this.basePath}/items`,
            data: createAttributes
        })
    }

    public deleteCartItem(id: number): Promise<PrescriptionCart> {
        return this.makeRequest({
            method: 'DELETE',
            params: {id},
            url: `${this.basePath}/items/{id}`
        })
    }

    public updateCartItem(updateAttributes: PrescriptionCartItemUpdateAttributes): Promise<PrescriptionCart> {
        return this.makeRequest({
            method: 'PUT',
            params: _.pick(updateAttributes, ['id']),
            url: `${this.basePath}/items/{id}`,
            data: _.pick(updateAttributes, ['quantity', 'directions'])
        })
    }

}
