import * as Promise from 'bluebird';

import {
    YLIFTCreateAttributes,
    YLIFTListAttributes,
    YLIFTUpdateAttributes,
    Resource
} from '../utils/Resource';
import { RequestManager } from '../RequestManager';
import {Sku, SkuRoleAttribute, SkuRoleAttributeUpdateAttributes} from './Sku';
import {ProductAttribute} from "./ProductAttribute";
import {ProductSubcategory} from "./ProductSubcategory";
import {ProductBrand} from "./ProductBrand";
import {PricingRule} from "./PricingRule";
import {ProductMedia} from "./ProductMedia";
import {TaxableProductCategory} from "./TaxableProductCategory";
import {ProductShippingSetting} from "./ProductShippingSetting";
import {Vendor} from "./Vendor";

export interface Product {
    id: number;
    createdAt: Date;
    updatedAt: Date;
    name: string;
    description: string|null;
    defaultSkuId: number|null;
    defaultSku?: Sku;
    isActive: boolean;
    vendorId: number|null;
    vendor?: Vendor;
    featured: boolean;
    taxableProductCategoryId: number|null;
    taxableProductCategory?: TaxableProductCategory;
    note: string|null;
    isShippable: boolean|null;
    isAdminApprovalRequired: boolean;
    prescriptionRequired: boolean;
    isLeasable: boolean;
    taxType: 'direct'|'flowthrough';
    skus?: Sku[];
    productAttributes?: ProductAttribute[];
    productBrands?: ProductBrand[];
    productSubcategories?: ProductSubcategory[];
    pricingRules?: PricingRule[];
    productMedia?: ProductMedia[];
    productShippingSetting?: ProductShippingSetting[];
    canApplyDiscount?: boolean;
}

export interface ProductRoleAttribute {
    id: number;
    createdAt: Date;
    updatedAt: Date;
    productId: number;
    role: String;
    isActive?: boolean;
}

export interface ProductRoleAttributeUpdateAttributes {
    productId: number;
    role: String;
    isActive?: boolean;
}

export interface ProductListAttributes extends YLIFTListAttributes {
    isActive?: boolean;
    vendorId?: number|null;
    featured?: boolean;
}

export interface ProductSearchAttributes extends YLIFTListAttributes {
    id?: number;
    query?: string;
    categories?: number[];
    subcategories?: number[];
    brands?: number[];
    filters?: string[];
    sort?: string;
    price?: string;
    ignoreRoleFilter?: boolean;
}

export interface ProductSearchSimilarAttributes extends YLIFTListAttributes {
    query?: string;
    id?: number|number[];
}

export interface ProductCreateAttributes extends YLIFTCreateAttributes {
    name: string;
    description?: string|null;
    defaultSkuId?: number|null;
    isActive?: boolean;
    vendorId?: number|null;
    featured?: boolean;
    taxableProductCategoryId?: number|null;
    note?: string|null;
    isShippable?: boolean|null;
    isAdminApprovalRequired?: boolean;
    prescriptionRequired?: boolean;
    isLeasable?: boolean;
    taxType?: 'direct'|'flowthrough';
}

export interface ProductUpdateAttributes extends YLIFTUpdateAttributes {
    name?: string;
    description?: string|null;
    defaultSkuId?: number|null;
    isActive?: boolean;
    vendorId?: number|null;
    featured?: boolean;
    taxableProductCategoryId?: number|null;
    note?: string|null;
    isShippable?: boolean|null;
    isAdminApprovalRequired?: boolean;
    prescriptionRequired?: boolean;
    isLeasable?: boolean;
    taxType?: 'direct'|'flowthrough';
}

export class ProductResource extends Resource<Product, ProductListAttributes, ProductCreateAttributes, ProductUpdateAttributes> {
    constructor(requestManager: RequestManager) {
        super('/products', requestManager);
    }

    public list(listArgs: ProductListAttributes): Promise<{ count: number, rows: Product[] }> {
        return this._list(listArgs);
    }

    public search(searchArgs: ProductSearchAttributes): Promise<{ count: number, rows: Product[], aggregations: any }> {
        return this.makeRequest({
            method: 'GET',
            params: searchArgs,
            url: `${this.basePath}/search`
        })
    }

    public suggest(query: string): Promise<{ suggestions: any[] }> {
        return this.makeRequest({
            method: 'GET',
            params: { query },
            url: `${this.basePath}/search/suggest`
        })
    }

    public similar(similarArgs: ProductSearchSimilarAttributes): Promise<{ count: number, rows: Product[] }> {
        return this.makeRequest({
            method: 'GET',
            params: similarArgs,
            url: `${this.basePath}/search/similar`
        })
    }

    public create(createArgs: ProductCreateAttributes): Promise<Product> {
        return this._create(createArgs);
    }

    public get(id: string | number): Promise<Product> {
        return this._get(id);
    }

    public update(id: string | number, updateArgs: ProductUpdateAttributes): Promise<Product> {
        return this._update(id, updateArgs);
    }

    public getRoleAttributes(productId: number): Promise<ProductRoleAttribute[]> {
        return this.makeRequest({
            method: 'GET',
            params: {productId},
            url: `${this.basePath}/role-attributes`
        })
    }

    public updateRoleAttribute(updateArgs: ProductRoleAttributeUpdateAttributes): Promise<SkuRoleAttribute[]> {
        return this.makeRequest({
            method: 'PUT',
            data: updateArgs,
            url: `${this.basePath}/role-attributes`
        })
    }

    public del(id: string | number): Promise<void> {
        return this.destroy(id);
    }

    public destroy(id: string | number): Promise<void> {
        return this._destroy(id);
    }
}
