import * as Promise from 'bluebird';
import * as _ from 'lodash';

import {
    YLIFTCreateAttributes,
    YLIFTListAttributes,
    YLIFTUpdateAttributes,
    Resource
} from '../utils/Resource';
import { RequestManager } from '../RequestManager';
import {Product} from "./Product";

export interface ProductAttribute {
    id: number;
    createdAt: Date;
    updatedAt: Date;
    name: string;
    productId: number;
    product: Product;
    isArchived?: boolean;
}

export interface ProductAttributeGetAttributes {
    id: string;
    productId: number;
}

export interface ProductAttributeListAttributes extends YLIFTListAttributes {
    productId: number;
    name?: string;
}

export interface ProductAttributeCreateAttributes extends YLIFTCreateAttributes {
    name: string;
    productId: number;
    isArchived?: boolean;
}

export interface ProductAttributeUpdateAttributes extends YLIFTUpdateAttributes {
    productId: number;
    name?: string;
    isArchived?: boolean;
}

export interface ProductAttributeDeleteAttributes {
    id: string;
    productId: number;
}

export class ProductAttributeResource extends Resource<ProductAttribute, ProductAttributeListAttributes, ProductAttributeCreateAttributes, ProductAttributeUpdateAttributes> {
    constructor(requestManager: RequestManager) {
        super('/products/{productId}/attributes', requestManager);
    }

    public list(listArgs: ProductAttributeListAttributes): Promise<{ count: number, rows: ProductAttribute[] }> {
        return this._list(listArgs);
    }

    public create(createArgs: ProductAttributeCreateAttributes): Promise<ProductAttribute> {
        return this.makeRequest({
            method: 'POST',
            params: _.pick(createArgs, ['productId']),
            data: _.omit(createArgs, ['productId']),
            url: `${this.basePath}`
        })
    }

    public get(getArgs: ProductAttributeGetAttributes): Promise<ProductAttribute> {
        return this.makeRequest({
            method: 'GET',
            params: getArgs,
            url: `${this.basePath}/{id}`
        })
    }

    public update(id: string | number, updateArgs: ProductAttributeUpdateAttributes): Promise<ProductAttribute> {
        return this.makeRequest({
            method: 'PUT',
            params: _.pick(updateArgs, ['id', 'productId']),
            data: _.omit(updateArgs, ['id', 'productId']),
            url: `${this.basePath}`
        })
    }

    public del(deleteArgs: ProductAttributeDeleteAttributes): Promise<void> {
        return this.makeRequest({
            method: 'DELETE',
            params: deleteArgs,
            url: `${this.basePath}/{id}`
        })
    }

    public destroy(deleteArgs: ProductAttributeDeleteAttributes): Promise<void> {
        return this.makeRequest({
            method: 'DELETE',
            params: deleteArgs,
            url: `${this.basePath}/{id}`
        })
    }
}
