import * as Promise from 'bluebird';

import {
    YLIFTCreateAttributes,
    YLIFTListAttributes,
    YLIFTUpdateAttributes,
    Resource
} from '../utils/Resource';
import { RequestManager } from '../RequestManager';

export interface Profile {
    id: number;
    createdAt: Date;
    updatedAt: Date;
    name: string|null;
    email: string;
    phone: string|null;
    timezone: string;
    isEmailConfirmed: boolean;
    lastLogin: Date|null;
    loginCount: number;
    lastActive: Date|null;
    customerId: string;
    parentProfileId: number|null;
}

export interface ProfileListAttributes extends YLIFTListAttributes {
    email?: string;
    name?: string;
    role?: string;
    customerId?: number;
    parentProfileId?: number;
}

export interface ProfileCreateAttributes extends YLIFTCreateAttributes {
    email: string;
    name?: string|null;
    phone?: string|null;
    timezone?: string;
    isEmailConfirmed?: boolean;
    customerId?: string;
    lastLogin?: Date|null;
    loginCount?: number;
    lastActive?: Date|null;
    parentProfileId?: number|null;
}

export interface ProfileUpdateAttributes extends YLIFTUpdateAttributes {
    email?: string;
    name?: string|null;
    phone?: string|null;
    timezone?: string;
    isEmailConfirmed?: boolean;
    customerId?: string;
    lastLogin?: Date|null;
    loginCount?: number;
    lastActive?: Date|null;
    parentProfileId?: number|null;
}

export interface CardCreateAttributes extends YLIFTCreateAttributes {
    profileId?: number;
    cardNumber: string;
    cardCvc: string;
    cardExpMonth: string;
    cardExpYear: string;
    addressFirstName: string;
    addressLastName: string;
    addressLine: string;
    addressCity: string;
    addressState: string;
    addressZip: string;
}

export interface ProfileUpdateEmailAttributes {
    email: string;
    profileId: number;
}

export interface PendingInviteRecord {
    id: string;
    profileId: number;
    email: string;
    createdAt: Date;
    updatedAt: Date;
}

export class ProfileResource extends Resource<Profile, ProfileListAttributes, ProfileCreateAttributes, ProfileUpdateAttributes> {
    constructor(requestManager: RequestManager) {
        super('/profiles', requestManager);
    }

    public list(listArgs: ProfileListAttributes): Promise<{ count: number, rows: Profile[] }> {
        return this._list(listArgs);
    }

    public create(createArgs: ProfileCreateAttributes): Promise<Profile> {
        return this._create(createArgs);
    }

    public me(): Promise<Profile> {
        return this.makeRequest({
            method: 'GET',
            url: `${this.basePath}/me`
        })
    }

    public role(): Promise<Profile> {
        return this.makeRequest({
            method: 'GET',
            url: `${this.basePath}/role`
        })
    }

    public get(id: string | number): Promise<Profile> {
        return this._get(id);
    }

    public update(id: string | number, updateArgs: ProfileUpdateAttributes): Promise<Profile> {
        return this._update(id, updateArgs);
    }

    public updateEmail(updateEmailArgs: ProfileUpdateEmailAttributes): Promise<Profile> {
        return this.makeRequest({
            method: 'PUT',
            url: `${this.basePath}/email`,
            data: updateEmailArgs
        });
    }

    public del(id: string | number): Promise<void> {
        return this.destroy(id);
    }

    public destroy(id: string | number): Promise<void> {
        return this._destroy(id);
    }

    public listCards(listArgs: { profileId?: number }) {
        return this.makeRequest({
            method: 'GET',
            params: listArgs,
            url: `${this.basePath}/cards`
        })
    }

    public createCard(createArgs: CardCreateAttributes) {
        return this.makeRequest({
            method: 'POST',
            data: createArgs,
            url: `${this.basePath}/cards`
        })
    }

    public deleteCard(cardId: string) {
        return this.makeRequest({
            method: 'DELETE',
            params: { cardId },
            url: `${this.basePath}/cards/{cardId}`
        })
    }

    public recruitUser(email: string) {
        return this.makeRequest({
            method: 'POST',
            data: { email },
            url: `${this.basePath}/recruit`
        })
    }

    public pendingInvites(profileId?: number): Promise<PendingInviteRecord[]> {
        return this.makeRequest({
            method: 'GET',
            params: { profileId },
            url: `${this.basePath}/invited`
        })
    }
}
