import * as Promise from 'bluebird';

import {
    YLIFTCreateAttributes,
    YLIFTListAttributes,
    YLIFTUpdateAttributes,
    Resource
} from '../utils/Resource';
import {RequestManager} from '../RequestManager';
import {Product} from "./Product";
import {ProductMedia} from "./ProductMedia";
import {Cart} from "./Cart";
import {PricingRule} from "./PricingRule";
import {SkuProductAttributeDescription} from './SkuProductAttributeDescription';

export interface Sku {
    id: number;
    createdAt: Date;
    updatedAt: Date;
    productId: number;
    product?: Product;
    price: number;
    defaultProductMediaId: number|null;
    defaultProductMedia?: ProductMedia;
    isActive: boolean|null;
    retailPrice: number|null;
    taxablePrice: number|null;
    isArchived: boolean|null;
    cost: number|null;
    isResellable: boolean;
    quantityIncrement: number;
    quantityMin: number;
    quantityMax: number;
    skuProductAttributeDescriptions: SkuProductAttributeDescription[];
}

export interface SkuRoleAttribute {
    id: number;
    createdAt: Date;
    updatedAt: Date;
    skuId: number;
    roleName: String;
    isActive?: boolean;
    price?: number;
}

export interface SkuListAttributes extends YLIFTListAttributes {
    productId?: number;
    isActive?: boolean;
}

export interface SkuCreateAttributes extends YLIFTCreateAttributes {
    productId: number;
    price: number;
    defaultProductMediaId?: number|null;
    isActive?: boolean|null;
    retailPrice?: number|null;
    taxablePrice?: number|null;
    isArchived?: boolean|null;
    cost?: number|null;
    isResellable?: boolean;
    quantityIncrement?: number;
    quantityMin?: number;
    quantityMax?: number;
}

export interface SkuUpdateAttributes extends YLIFTUpdateAttributes {
    productId?: number;
    price?: number;
    defaultProductMediaId?: number|null;
    isActive?: boolean|null;
    retailPrice?: number|null;
    taxablePrice?: number|null;
    isArchived?: boolean|null;
    cost?: number|null;
    isResellable?: boolean;
    quantityIncrement?: number;
    quantityMin?: number;
    quantityMax?: number;
}

export interface SkuRoleAttributeUpdateAttributes extends YLIFTUpdateAttributes {
    skuId: number;
    roleName: String;
    isActive?: boolean;
    price?: number;
}

export class SkuResource extends Resource<Sku, SkuListAttributes, SkuCreateAttributes, SkuUpdateAttributes> {
    constructor(requestManager: RequestManager) {
        super('/products/skus', requestManager);
    }

    public list(listArgs: SkuListAttributes): Promise<{ count: number, rows: Sku[] }> {
        return this._list(listArgs);
    }

    public create(createArgs: SkuCreateAttributes): Promise<Sku> {
        return this._create(createArgs);
    }

    public get(id: string | number): Promise<Sku> {
        return this._get(id);
    }

    public getPricingRules(skuId: number): Promise<PricingRule[]> {
        return this.makeRequest({
            method: 'GET',
            params: {skuId},
            url: `${this.basePath}/{skuId}/pricing-rules`
        })
    }

    public getRoleAttributes(skuId: number): Promise<SkuRoleAttribute[]> {
        return this.makeRequest({
            method: 'GET',
            params: {skuId},
            url: `${this.basePath}/role-attributes`
        })
    }

    public update(id: string | number, updateArgs: SkuUpdateAttributes): Promise<Sku> {
        return this._update(id, updateArgs);
    }

    public updateRoleAttribute(updateArgs: SkuRoleAttributeUpdateAttributes): Promise<SkuRoleAttribute[]> {
        return this.makeRequest({
            method: 'PUT',
            data: updateArgs,
            url: `${this.basePath}/role-attributes`
        })
    }

    public del(id: string | number): Promise<void> {
        return this.destroy(id);
    }

    public destroy(id: string | number): Promise<void> {
        return this._destroy(id);
    }
}
